/*	Bars.h
*/

/*
	modified 17 Jan 93 mwr to include prototypical function declarations in
   functions structure.
   and 23 Jun 93 to include changed prototypes and additional functions lifted
   from bppro.h (v2.0).
	and again 8 Feb 94 to include stuff from v.h from mark
	and 7 Apr 94 to add mflag field to struct Chord
	and 7 Sep 94 to add ROPE_LENGTH flag
	and 11 Nov 96 to add __stdargs to function declarations
*/

#ifndef EXEC_TYPES_H
 #include <exec/types.h>
#endif

#define MAXTEMPO 350
#define MINTEMPO  10

#define MIDI_NOTEOFF	  0x80
#define MIDI_NOTEON	  0x90
#define MIDI_PTOUCH	  0xA0
#define MIDI_CCHANGE	  0xB0
#define MIDI_PCHANGE	  0xC0
#define MIDI_MTOUCH	  0xD0
#define MIDI_PBEND     0xE0
#define MIDI_SYSX	     0xF0
#define MIDI_MTC       0xF1
#define MIDI_SONGPP	  0xF2
#define MIDI_SONGS	  0xF3
#define MIDI_EOX       0xF7
#define MIDI_CLOCK	  0xF8
#define MIDI_START	  0xFA
#define MIDI_CONTINUE  0xFB
#define MIDI_STOP      0xFC
#define MIDI_SENSE     0xFE

#define EVENT_ONTIME   0x10   /* This event is an on-time event. */
#define EVENT_BRANCH   0x20   /* This event is traversing a branch. */
#define EVENT_PADEDIT  0x40   /* Not a real time event. */
#define EVENT_PLAYONLY 0x80   /* Discard event when recording. */
#define EVENT_VOICE    1      /* Performance event */
#define EVENT_SYSX     2      /* System exclusive event. */
#define EVENT_LYRIC    3      /* Lyric string. */
#define EVENT_TIMESIG  4      /* Time signature change event. */
#define EVENT_KEY	     5      /* Key change event. */
#define EVENT_CHORD	  6      /* Chord change event. */
#define EVENT_RHYTHM	  7      /* Rhythm template event. */
#define EVENT_DYNAMICS 8      /* Dynamics event. */
#define EVENT_SEED	  9      /* Use to seed repeating tools. */

/* in edit.h 
enum eRawkeys
{
	RK_1 = 0x01, RK_2, RK_3, RK_4, RK_5, RK_6, RK_7, RK_8, RK_9, RK_0,
	RK_Q = 0x10, RK_W, RK_E, RK_R, RK_T, RK_Y, RK_U, RK_I, RK_O, RK_P,
	RK_A = 0x20, RK_S, RK_D, RK_F, RK_G, RK_H, RK_J, RK_K, RK_L,
	RK_Z = 0x31, RK_X, RK_C, RK_V, RK_B, RK_N, RK_M, RK_LESS, RK_GREATER, RK_SLASH
};
*/

struct Event {
    struct Event *next;   /* The next event in the list. */
    long time;            /* When this event occurs. */
    char type;            /* What type of event. */
    unsigned char status; /* MIDI status. */
    unsigned char byte1;  /* First byte of data. */
    unsigned char byte2;  /* Second byte of data. */
    long data;            /* Data storage. */
    struct Tool *tool;    /* Tool that processes this next. */
};

struct EventList {
    struct Event *first;		/* First in list. */
    struct Event *point;		/* Current position in list. */
};


struct NoteEvent {
    struct NoteEvent *next;	/* The next event in the list. */
    long time;						/* When this event occurs. */
    char type;						/* What type of event. */
    unsigned char status;		/* MIDI status. */
    unsigned char value;		/* Note value. */
    unsigned char velocity;	/* Note velocity. */
    unsigned short duration;	/* The duration of this event. */
    short data;					/* Data storage. */
    struct Tool *tool;			/* Tool that processes this next. */
};

struct String {
    unsigned short length;	/* The length of the string that follows. */
    char string[1];			/* This is actually a variable length array. */
};

struct StringEvent {
    struct StringEvent *next;	/* The next event in the list. */
    long time;						/* When this event occurs. */
    char type;						/* What type of event. */
    unsigned char status;		/* For SysEx. */
    short length;					/* Display length. */
    struct String *string;		/* Pointer to string. */
    struct Tool *tool;			/* Tool that processes this next. */
};

struct SysEx {
    unsigned long length;	/* The length of the packet. */
    struct Task *task;		/* Sending task, for signaling. */
    short signal;				/* Signal number. */
    char done;					/* Done flag. */
    char *data;				/* The data. */
};

struct SysExEvent {
    struct SysExEvent *next;	/* The next event in the list. */
    long time;						/* When this event occurs. */
    char type;						/* What type of event. */
    unsigned char status;		/* For SysEx. */
    short length;					/* Display length. */
    struct SysEx *sysex;		/* Pointer to data. */
    struct Tool *tool;			/* Tool that processes this next. */
};

struct Chord {
    struct Chord *next;		/* Next in list. */
    struct String *name;	/* Chord name. */
    unsigned long pattern;	/* Bit pattern of chord. */
    unsigned short id;		/* Chord identifier. */
    char count;				/* Number of notes in chord. */
};

struct ChordEvent {
    struct ChordEvent *next;	/* The next event in the list. */
    long time;						/* When this event occurs. */
    char type;						/* What type of event. */
    unsigned char root;			/* Root note. */
    char flats;					/* Chord root is a flat. */
    char pad;
    unsigned long chordid;		/* Chord id. */
    struct Chord *chord;		/* Chord. */
};

struct KeyEvent {
    struct KeyEvent *next;		/* The next event in the list. */
    long time;				/* When this event occurs. */
    char type;				/* What type of event. */
    unsigned char root;			/* Root note. */
    char flats;				/* Display the key with flats. */
    char pad;
    unsigned long scaleid;		/* Scale id. */
    struct Chord *scale;		/* Pointer to Scale definition. */
};


struct TimeSigEvent {
    struct TimeSigEvent *next;		/* The next event in the list. */
    long time;				/* When this event occurs. */
    char type;				/* What type of event. */
    unsigned char beatcount;		/* Number of beats per measure. */
    unsigned char beat;			/* What note gets beat. */
    char pad;
    unsigned short clocks;    		/* Clocks per beat. */
    unsigned short measures;   		/* Measures till next. */
    unsigned long totalclocks;		/* Clocks till next. */
};

struct TimeSigList {
    struct TimeSigEvent *first;		/* First in list. */
    struct TimeSigEvent *point;		/* Current position in list. */
    unsigned short measure;		/* Current measure. */
    unsigned short beat;		/* Current beat. */
    unsigned short clock;		/* Current clock. */
};

struct Rhythm {
    struct Rhythm *next;		/* Next in list of templates. */
    struct String *name;		/* Name of this rhythm. */
    struct NoteEvent *notes;		/* List of notes. */
    long length;			/* Loop length. */
    short id;				/* Identifier. */
};

struct RhythmEvent {
    struct RhythmEvent *next;		/* The next event in the list. */
    long time;				/* When this event occurs. */
    char type;				/* What type of event. */
    char pad;				/* Empty. */
    unsigned short rhythmid;		/* Id of rhythm clip. */
    struct Rhythm *rhythm;		/* Rhythm. */
    struct NoteEvent *point;		/* Current position in rhythm. */
};

struct DynamicsEvent {
    struct DynamicsEvent *next;	/* The next event in the list. */
    long time;							/* When this event occurs. */
    char type;							/* What type of event. */
    unsigned char value;			/* Dynamic value. */
    unsigned char lastvalue;		/* Previous dynamic value. */
    unsigned char nextvalue;		/* Next Dynamic value. */
    long lasttime;					/* Previous time. */
    long nexttime;					/* Next time. */
};

/* mwr added 20 Jul 93 */
#ifndef CLIP_GET
 #define CLIP_GET         1
 #define CLIP_PUT         2
#endif

#ifndef CLIP_OPEN
 #define CLIP_OPEN        4
 #define CLIP_CLOSE       8
 #define CLIP_SHUTDOWN 0x80
 #define CLIP_OK          1
 #define CLIP_EMPTY       2
#endif


struct Clip {
    struct Clip *next;				/* List of clips. */
    struct EventList events;		/* Event list. */
    struct EventList chords;		/* List of bass chords. */
    struct EventList keys;			/* List of keys. */
    struct EventList lyrics;		/* List of lyrics. */
    struct EventList rhythm;		/* List of rhythm templates. */
    struct EventList dynamics;	/* List of dynamic changes. */
    struct TimeSigList timesig;	/* List of time signatures. */
    struct String *name;			/* Name of this clip. */
    struct String *notes;			/* Notes for this clip. */
    long begin;						/* Time this begins. */
    long end;							/* Time this ends. */
    unsigned char highnote;		/* Highest note, for display. */
    unsigned char lownote;			/* Lowest note, for display. */
    char locked;						/* Locked during record. */
};
 
/*	For handling a linked list of clips in parallel: */

struct ClipList {
    struct ClipList *next;		/* Next Clip or ClipList. */
    long cliplisttag;			/* Tag to identify ClipList. */
    struct Clip *cliplist;		/* List of clips. */
};


struct Tool {
    struct Tool *next;					/* Next tool used by this track. */
    struct Tool *branch;				/* Tool on other track. */
    struct Tool *parent;				/* Parent tool (for macros.) */
    struct ToolMaster *toolmaster;	/* Pointer to actual tool. */
    struct Clip *clip;					/* Clip to be worked on. */
    struct String *name;				/* Name of this tool. */
    struct Window *window;				/* Edit window. */
    struct Track *track;				/* owning Track; NULL if in ToolPad (usually) */
    long toolid;							/* Tool ID. */
    unsigned short left,top;			/* Position of edit window. */
    unsigned short width,height;		/* Size of edit window. */
    unsigned short x,y;					/* Position in pipe display. */
    unsigned short xindex;				/* How far down list this is. */
    unsigned short yindex;				/* How far down track list this is. */
    short branchindex;					/* How far away branch tool is. */
    unsigned short id;					/* ID for file io. */
    char intool;							/* True if inlist, false if outlist. */
    char inedit;							/* Flag to indicate editing now. */
    char touched;							/* This tool has been edited. */
    char selected;						/* Icon selected in graph. */
    long tooltype;						/* Sequence? Input? Branch? */
	 /* More tool unique stuff here... */
};

#define TOUCH_EDIT		1	/* Tool has been edited. */
#define TOUCH_INIT		2	/* Tool has been initialised. */

#define TOOL_SEQUENCE        1   /* This is actually the track. */
#define TOOL_INPUT           2   /* This is an input tool. */
#define TOOL_OUTPUT          4   /* This is an output tool. */
#define TOOL_NORMAL          8   /* This is a normal tool. */
#define TOOL_ONTIME       0x10   /* This tool doesn't accept early events. */
#define TOOL_BRANCHIN     0x20   /* This tool merges two inputs. */
#define TOOL_BRANCHOUT    0x40   /* This tool has two outputs. */
#define TOOL_MACRO        0x80   /* This tool is a macro tool. */
#define TOOL_MACROOUT    0x100   /* This is the output of macro. */
#define TOOL_MACROBRANCH 0x200   /* This is the branch output of macro. */
#define TOOL_MACROIN		 0x400   /* This is the input of macro. */
#define TOOL_GROUPIN		 0x800   /* This tool part of group input. */
#define TOOL_MIDI		   0x1000   /* This tool is MIDI capable. */
#define TOOL_NOTPAD     0x2000   /* This can not go in the toolpad. */
#define TOOL_STOOL      0x4000   /* This is a sequencer tool. */
#define TOOL_NOTPIPE    0x8000   /* This tool can not go in pipeline. */

#define MMCMD_COPYALLFILES      0x0001  /* pass the path name to copy to */
#define MMCMD_INSTALLHITLIST    0x0002  /* no parameters */
#define MMCMD_CONVERTTOHIRES    0x0003  /* no parameters */
#define MMCMD_CONVERTFROMHIRES  0x0004  /* no parameters */

struct ToolMaster {
    struct ToolMaster *next;				/* Next tool in this list. */
    long toolid;								/* Tool ID. */
    struct Image *image;					/* Icon for this tool. */
    struct Image *upimage;					/* Icon for branching up. */
    short x,y;									/* Position in toolbox for display. */
    char name[100];							/* Tool name. */
    char filename[100];						/* Where it is stored on disk. */
    struct Tool *(* __stdargs createtool)();		/* Routine to allocate a new tool. */
    void (* __stdargs edittool)();					/* Routine to edit tool parameters. */
    struct Event *(* __stdargs processevent)();	/* Routine to process an event. */
    struct Event *(* __stdargs processlist)();	/* Routine to process a list of events. */
    void (* __stdargs deletetool)();				/* Routine to delete a tool. */
    void (* __stdargs removetool)();				/* Routine to close down. */
    long (* __stdargs savesize)();					/* Returns size prior to save. */
    long (* __stdargs savetool)();					/* Routine to save to disk. */
    struct Tool *(* __stdargs loadtool)();		/* Routine to load from disk. */
    void (* __stdargs readsysex)();					/* Pass sysex byte */
    long (* __stdargs multimediafunc)();        /* passed tool ptr,(long)command,
                                                   variable params. */
    void (* __stdargs drawtoolicon)();          /* pass tool ptr,rastport,x,y,wid,height */
    long segment;								/* This tool's segment list. */
    long altsegment;							/* For later.. */
    struct Track *intrack;					/* Input track for this tool. */
    short toolsize;							/* Tool size for loading and saving. */
    char inedit;								/* Flag to indicate editing now. */
    char selected;							/* Icon selected in graph. */
    long tooltype;							/* Type of tool. */
};

#define ID_MDIN	0x4D49494E
#define ID_MDOT	0x4D494F54
#define ID_MGIN	0x4D47494E
#define ID_MGOT 0x4D474F54
#define ID_PLUG 0x504C5547
#define ID_ELBO 0x454C424F
#define ID_MCOT 0x4D434F54
#define ID_MCBR 0x4D434252


struct MacroLine {
    struct Tool *toollist;		/* List of tools. */
    struct Tool *point;			/* For making display. */
    unsigned long marks[10];	/* For figuring out display. */
    short markindex;				/* Farthest right bit in marks. */
};

struct MacroTool {
    struct Tool tool;					/* Tool definition. */
    struct Tool *intool;				/* Tool at start of macro. */
    struct MacroLine macroline[6];	/* Array of macrolines. */
    short inline;							/* Which line input goes to. */
    short toolsleft;						/* Left edge of tool list. */
    short toolswidth;					/* Width of tools. */
    short toolsindent;					/* Indentation into tools. */
};

struct MacroMaster {
    struct ToolMaster master;	/* Tool Master structure. */
    struct MacroTool *macro;	/* The macro tool. */
    char saved;					/* Latest version saved. */
};


typedef struct ToolTrayItem {
    struct ToolTrayItem *next;
    char name[30];
    struct Tool *tool;
    short x,y;
} ToolTrayItem;

typedef struct ToolTray {
    struct ToolTray *next;
    char name[16];
    ToolTrayItem *list;
    ToolTrayItem *selected;
    struct Window *window;
} ToolTray;

/*	For display purposes, mark a measure as in use. */

struct Measure {
    struct Measure *next;
    long time;
    char type;
    char inuse;
    short length;
    struct Track *track;
    short x;
    short y;
};

#define HITNAMESIZE	30

struct HitName {
    struct HitName *next;		/* Linked list. */
    unsigned char note;			/* Equivalent MIDI value. */
    char name[HITNAMESIZE];		/* Text. */
};

typedef struct PatchNames {
    struct PatchNames *next;		/* Next in list. */
    char name[20];			/* Name of list. */
    short usecount;			/* # of tools using this. */
    unsigned char names[128][14];	/* Array of names. */
} PatchNames;

struct Mix {
    struct Tool ontimetool;				/* Tool to do realtime display. */
    struct NoteEvent *volumelist;		/* List of new volumes. */
    struct NoteEvent *panlist;			/* List of new pans. */
    struct NoteEvent *volumeonofflist;	/* Time stamps - when user dragged. */
    struct NoteEvent *panonofflist;		/* Pan time stamps. */
    unsigned char volume;					/* Currently displayed volume. */
    unsigned char newvolume;				/* Desired volume for display. */
    unsigned char pan;						/* Currently displayed pan. */
    unsigned char newpan;					/* Desired pan for display. */
    unsigned char velocitydown[32];		/* Velocities of held notes. */
    unsigned char meter;					/* Currently displayed meter. */
    unsigned char newmeter;				/* Desired meter setting. */
    short totalvelocity;					/* Accumulated velocities. */
    char draggingvol;						/* Volume is being dragged. */
    char draggingpan;						/* Pan is being dragged. */
    char locked;								/* Lock button in MixMaestro. */
    char displayed;							/* This is displayed. */
    short leftedge;							/* Position in display. */
    short id;									/* Gadget ID base. */
};
    
struct Track {
    struct Track *next;			/* Next Track in the list. */
    struct Edit *edit;			/* Edit structure for this track. (see edit.h) */
    struct Clip clip;			/* A clip that defines the sequence. */
    struct Clip cut;				/* A clip for global editing. */
    struct Clip undo;			/* A clip for undo command. */
    struct EventList record;	/* Event list for recording. */
    struct Tool *toollist;		/* List of tools. */
    struct Tool tool;			/* Sequence tool. */
    struct Tool *point;			/* For making display. */
    unsigned char channelin;	/* MIDI Channel coming in. */
    unsigned char channelout;	/* MIDI Channel going out. */
    unsigned char mode;			/* Mute | Through | Record. */
    unsigned char selected;	/* Track is highlighted. */
    long group;					/* Group bits. */
    unsigned long marks[20];	/* For figuring out display. */
    short markindex;				/* Farthest right bit in marks. */
    short height;					/* Height in display. */
    short nameleft;				/* Left edge of name. */
    short namewidth;				/* Width of name field. */
    short nameindent;			/* Indentation into name field. */
    short channelinleft;		/* Left edge of channel in. */
    short toolsinleft;			/* Left edge of toolsin. */
    short toolsinwidth;			/* Width of toolsin field. */
    short toolsinindent;		/* Indentation into toolsin field. */
    short playrecordleft;		/* Left edge of play/record. */
    short sequenceleft;			/* Left edge of sequence. */
    short sequencewidth;		/* Width of sequence field. */
    long sequenceindent;		/* Indentation into sequence field. */
    short muteleft;				/* Left edge of mute/through. */
    short toolsoutleft;			/* Left edge of toolsout. */
    short toolsoutwidth;		/* Width of toolsout field. */
    short toolsoutindent;		/* Indentation into toolsout field. */
    short toolsouttotalwidth;	/* Total width of tools out. */
    short channeloutleft;		/* Left edge of channel out. */
    struct Measure *measures;	/* List of active measures. */
    struct Mix mix;				/* MixMaestro stuff. */
    long bits;						/* Various flags. */
    struct HitName *hitnamelist;	/* List of hit names. */
    PatchNames *patchnames;		/* Array of patch names. */
};

#define TB_NOUNDO	1		/* This track has no undo buffer. */
#define TB_PRINTEDIT	2	/* This track has a false edit structure. */
#define TB_STOOL	   4	/* There's a sequence tool for this track. */

#define TRACK_MUTE		 1
#define TRACK_THROUGH	 2
#define TRACK_RECORD		 4
#define TRACK_SOLOMUTE	 8		/* Track was muted for solo. */
#define TRACK_INEDIT		16
#define TRACK_REALTIME	32
#define TRACK_MERGE		64		/* Merge while recording. */

struct Song {
    struct Song *next;
    long time;				/* Start time in clocks. */
    long frametime;			/* Start time in frames. */
    long length;			/* Length in clocks. */
    long frameend;			/* End time in frames. */
    struct TempoChange *changelist;	/* Tempo map for this section. */
    struct Tempo *tempolist;		/* Resulting Tempo list. */
    struct Clip masterclip;		/* Song parameters. */
    struct Section *sectionlist;	/* ABA stuff. */
    short inittempo;			/* Initial tempo. */
    short id;
    char name[100];
    short color;			/* Display color. */
    short y;				/* Display coordinates. */
};

struct Accessory {
    struct Accessory *next;		/* Next accessory in this list. */
    long id;				/* Accessory ID. */
    struct Image *image;		/* Icon for this accessory. */
    struct Image *onimage;		/* Icon for when selected. */
    char name[100];			/* Name. */
    char filename[100];			/* Where it is stored on disk. */
    struct Window *window;
    unsigned short left,top;		/* Position of edit window. */
    unsigned short width,height;	/* Size of edit window. */
    unsigned short x,y;			/* Position in access box. */
    long (* __stdargs remove)();
    long (* __stdargs edit)();			/* Routine to edit accessory. */
    long (* __stdargs open)();			/* Routine to open it. */
    long (* __stdargs close)();
    long (* __stdargs size)();
    long (* __stdargs save)();
    long (* __stdargs load)();
    long (* __stdargs install)();			/* Install in environment. */
    long (* __stdargs clear)();			/* Clear from environment. */
    long (* __stdargs expandc)();			/* Future routine? */
    long segment;			/* This accessory's segment list. */
    long altsegment;			/* Alternate segment list. */
    char selected;			/* Icon selected in graph. */
};

#define TC_START	1
#define TC_STOP		2
#define TC_POSITION	3
#define TC_RECORDON	4
#define TC_RECORDOFF	5
#define TC_PLAY		6
#define TC_TICK		7

struct Transport {
    struct Transport *next;
    void (* __stdargs routine)();
    char priority;
    char loopflag; /* set if used loop transport routine */
    char pad[2];
    long offset;
};


/*	There is a linked list of Tempo nodes.  Each specifies a clock cycle,
	buffer size, and repeat count.  The last cycles forever.
*/

struct Tempo {
    struct Tempo *next;		/* Next Tempo in list. */
    long time;					/* Time, in clocks, this starts. */
    unsigned short cycle;	/* Sample rate. */
    unsigned short bpm;		/* Beats per minute. */
    long repeat;				/* Repeat count. */
    unsigned long frame;	/* Frame this starts on. */
    long offset;				/* offset for repeats */
};

/*	For editing the Tempo, the user sets Tempo Changes. */

struct TempoChange {
    struct TempoChange *next;
    long time;             /* When this occurs, in clicks. */
    char eventtype;        /* What type of event. */
    char type;             /* Type of change. */
    short bpm;             /* Beats per minute. */
    long duration;         /* How long the change should take. */
    struct Tempo *tempos;  /* Pointer into list of tempos. */
    char conform_paccess;  /* byte for use by Conform.paccess */
//    char more[3];          /* room for more */
    char more;
	 short end_bpm;  /* ending tempo for a non-instant change  mwr 31 Oct 95 */
};

#define TC_INSTANT  1
#define TC_LINEAR   2
#define TC_EXPCURVE 3
#define TC_LOGCURVE 4

struct SMPTE {
    unsigned char hours, minutes, seconds, frames;
};

struct Section {
    struct Section *next;
    long time;
    char type;				/* What type of event. */
    char pad[3];	/* in use in song.c, aba.c */
    struct String *string;		/* Pointer to string. */
    long length;			/* Length of this section. */
    char pattern;
    unsigned char repeat_count;		/* how many times to repeat (255 is infinite) */
    unsigned char current_repeat;	/* tracks the current repeat count */
    char c_more;	/* room for more */
};

/* 	Print flags */

#define PRINT_SECTIONS		 1
#define PRINT_BARNUMBERS	 2
#define PRINT_TEMPO		    4
#define PRINT_CHORDS			 8
#define PRINT_LYRICS		 0x10
#define PRINT_DYNAMICS	 0x20
#define PRINT_CONCERT	 0x40
#define PRINT_TRANSPOSED 0x80
#define PRINT_HIRES		0x100
#define PRINT_BLOWUP		0x200

/* 	Flag Bits. */

#define FLAGS_TWOPLANES            1	/* Two colors instead of three. */
#define FLAGS_SIMPLEREFRESH        2	/* Simple refresh windows. */
#define FLAGS_DEMOMODE             4	/* Currently in demo mode. */
#define FLAGS_COMBINETEMPO         8	/* Combine Timeline tempo maps. */
#define FLAGS_COMBINESECTIONS   0x10	/* Combine Timeline section lists. */
#define FLAGS_COMBINEPARAMS     0x20	/* Combine Timeline song params. */
#define FLAGS_DISABLERAWKEYS    0x40	/* Disable rawkeys for menu rawkeys */
#define FLAGS_SAVEICONS         0x80	/* Save file icons. */
#define FLAGS_WINDOWTOFRONT    0x100	/* Automatic Window to front. */
#define FLAGS_DOFLASH          0x200	/* Exproding windows. */
#define FLAGS_WBPOINTER        0x400	/* Use Workbench pointer. */
#define FLAGS_LACE             0x800	/* Run in interlace. */
#define FLAGS_UNDOBUFF        0x1000	/* Enable undo buffer. */
#define FLAGS_CLOSEWB         0x2000	/* Close WorkBench. */
#define FLAGS_USEASL          0x4000	/* use 2.0 asl requester */
#define FLAGS_DOUBLEWIDTH     0x8000	/* 2.0 double width screen. */
#define FLAGS_DOUBLEHEIGHT   0x10000	/* 2.0 double height screen. */
#define FLAGS_WBDISPLAY      0x20000	/* 2.0 copy workbench display. */

/*	Save flags. (long functions->saveflags)*/

#define SFLAG_TOOLNAMES        1	/* Display tools with names. */
#define SFLAG_LOCKTOMAP        2	/* Lock to tempo map. */
#define SFLAG_SMPTEOFFSET      4	/* Display SMPTE offset. */
#define SFLAG_MIDIDEFAULTS     8	/* Transmit MIDI defaults */
#define SFLAG_PIANOAUTORANGE  16	/* piano roll auto range on/off */
#define SFLAG_DONTALPHABETIZE 32	/* alphabetize tools */

#define RES_MEASURE  0
#define RES_BEAT     1
#define RES_ANYWHERE 2
#define RES_SECOND   3
#define RES_FRAME    4

typedef struct ToolName {
    struct ToolName *next;
    long toolid;
    char name[40];			/* Name. */
    char filename[100];		/* Where it is stored on disk. */
} ToolName;


/* lifted from rope.h 9 Feb 94 */

#include <graphics/text.h>

/* none of these flags are supported yet except ROPE_ACTIVE which is
   internal
*/

#define ROPE_LONGINT     0x0001
#define ROPE_CENTER      0x0002
#define ROPE_RIGHT       0x0004
/* Mark gave me this flag clandestinely. It makes the gadget deal with
 * a length of time (like for TimeWarp) instead of elapsed time. Call
 * InitRopeGadget(), set ROPE_LENGTH, then SetRopeInfo() makes it so. */
#define ROPE_LENGTH      0x0008
#define ROPE_LEFT        0x0000
#define ROPE_ACTIVE      0x8000
#define ROPE_USERTOGGLE1 0x1000
#define ROPE_USERTOGGLE2 0x2000

/* gadget->UserData points at struct RopeGadget after InitRopeGadget() */
typedef struct RopeGadget {
  void *UserData;
  char  onbackcolor, ontextcolor;
  char  offbackcolor,offtextcolor;
  short maxchars;
  short next_gad, prev_gad;
  char *buffer,*undobuffer;
  long  flags;
/* this stuff below is internal only */
  struct TextFont *font;
  short disppos,bufferpos;
  long  longint;
} RopeGadget;


/*	Array of pointers to shared data, functions and library pointers,
	so all tools can share these. */

/* this Functions structure is lifted from bpdev:rules/yak+2.0/bppro.h */
/* and updated after getting 2.1.2 v.h from mark */

struct Functions {
    char locked;
    char measureres;			/* Cuts resolve to measures. */
    char recording;			/* Set when recording. */
    char running;				/* Set when running. */
    char punchenabled;		/* Auto punch in and out enabled. */
    char loopenabled;		/* Loop mode turned on. */
    char clicking;			/* Click track on? */
    char seeclick;			/* Visual metronome. */
    char multiin;				/* Multiple inputs? */
    char clickchannel;		/* MIDI CLick channel. */
    char midiclock;			/* Sync to MIDI clocks. */
    char smpteclock;			/* Sync to SMPTE. */
    char sendmidiclock;		/* Send out MIDI clocks. */
    char smptetype;			/* Which SMPTE format. */
    char countdown;			/* Do count down. */
    char midiclick;			/* Use MIDI for click track. */
    char chop;
    long leadinstart;			/* Length of lead in. */
    long timenow;					/* Current time. */
    unsigned long markone;		/* Auto locate register. */
    unsigned long marktwo;		/* Auto locate register. */
    unsigned long punchin;		/* Punch in point. */
    unsigned long punchout;	/* Punch out point. */
    unsigned long loopin;		/* Loop in point. */
    unsigned long loopout;		/* Loop out point. */
    unsigned long cutin;		/* Cut in point. */
    unsigned long cutout;		/* Cut out point. */
    long starttime;           /* Where to play from. */
    long stoptime;				/* Marker to stop at. */
    unsigned long padcutin;	/* For non real time edits. */
    unsigned long padcutout;	/* For non real time edits. */
    unsigned long songlength;
    long startoffset;			/* Starting hi res clock offset. !smpte offset */
    unsigned short tempos[4];
    unsigned short tempo;		/* Current tempo. */
    unsigned short inittempo;	/* Initial tempo. */
    char songname[100];
    char author[100];
    short palette[8];			/* Colors. */
    char remotecontrol[128];	/* Table of remote controls. */
    unsigned long markthree;	/* Auto locate register. */
    unsigned long markfour;	/* Auto locate register. */ 
    unsigned char volumenum;	/* Mix Maestro Volume CC. */
    unsigned char pannum;		/* Mix Maestro Pan CC. */
    unsigned char subdivide;	/* Metronome subdivision. */
    char bypassmix;			   /* Use undo buffer. */
    long savestop;			   /* Place to save stop sign. */
    long printflags;			   /* Print flags. */
    unsigned long recordfilters;
    long userlength;		      /* User defined song length. */
    long saveflags;
    char clickvolume;			/* metronome volume */
    char cmore[3];			/* Expansion space. */
    long more;			/* Expansion space. */
    USHORT printbuf_x, printbuf_y;  /* size of print buffer */
    long time_offset;			/* current time offset taking into regard repeats */
    struct Tool *midiouttool;			/* Tool to send MIDI Metronome. */
    struct Tool *clockouttool;		/* Tool to send MIDI Clocks. */
    struct ToolTray *tooltraylist;	/* List of tool trays. */
    struct PatchNames *patchlist;	/* List of patch names lists. */
    struct Song *songlist;				/* For time-line stuff. */
    unsigned long clockcycles;		/* Time in clock cycles! */
    char useclip;			         /* Use the ClipBoard. */
    char externclock;				/* External clock flag. */
    short timeroffset;				/* For external clock. */
    struct Event *coordlist;		/* Window coordinates. */
    struct Track *tracklist;		/* Top track in list. */
    struct Clip masterclip;		/* Master key, chord, signature. */
    struct Clip masterundo;
    struct Clip mastercut;
    struct Edit *masteredit;    
    struct Tool *edittools[16];	/* 16 Tools to edit with. */
    unsigned short toolid;			/* Global tool id. */ 
    short groupid;					/* Currently selected group. */
    struct Chord *scalelist;		/* All scales. */
    struct Chord *chordlist;		/* All chords. */
    struct Rhythm *rhythmlist;	/* All rhythms. */
    struct Section *sectionlist;	/* ABA list. */
    struct TempoChange *tempochangelist;/* Tempo Change list. */
    long whoops;					/* Time line song list used to be here. */
    long frame;					/* Current frame. */
    unsigned long hirestime;	/* Hi res clock. */
    char version;
    long SysBase;			/* Exec library. */
    long DOSBase;			/* DOS library. */
    long IntuitionBase;
    long GfxBase;
    long LayersBase;
    long standardout;			      /* For printf. */
    struct Event *     pipequeue;	/* Event queue. */
    struct Event *     earlyqueue;	/* Ahead of time queue. */
    struct Screen *    screen;		/* Screen we all exist in. */
    struct Window *    window;		/* Main window. */
    struct ToolMaster *toolmasterlist;	/* All ToolMasters. */
    struct Accessory * accesslist;		/* All Accessories. */
    struct Tool *wasmidiouttool;	   /* Tool to send MIDI Metronome. */
    long (* __stdargs stealmidi)(void);	/* Steal serial interrupt. */
    long (* __stdargs releasemidi)(void);	/* Release MIDI. */
    long (* __stdargs audioclick)(void);	/* Play click sound. */

    struct Track *       selectedtrack;	/* Track currently selected. */
    long                 flags;				/* Additional flags. */
    char *               filename;			/* Pointer to filename buffer. */
    long                 sunrizeflags;
    struct DisplayFlags *displayflags;	   /* Display flags for updateobject. */
    short displaywidth,  displayheight;
    char                 osbits;			   /* Used by osms output. */
    char                 mmrecord;		   /* Enable mm record. */
/*** new stuff 8feb94 */
    unsigned short   CAMD_count;    /* count of open CAMD tools/accessories */
    struct MidiNode *CAMD_Node;     /* midi node for CAMD */
    char *           bppdir;		   /* directory&path BPP was run from */
    long *           lockdir;		   /* Lock to bppdir */
    struct Window *  windowtofront; /* if this has a value, main.c brings it to front */
    long             pad[39];			/* Room for more. */

    int (* __stdargs HandleRopeGadget)(struct Window *win,struct Gadget *gad,
	                                    short mouse_x, short mouse_y);
    void    (* __stdargs SetRopeInfo)(struct Window *win,short gadnum,char *text);
    void (* __stdargs InitRopeGadget)(struct Window *win,struct Gadget *gad,
                                      short next_gad,short prev_gad);
    void (* __stdargs FreeRopeGadget)(struct Gadget *gad);
    /* installtransportpl provides an extra offset value for repeating/
     * looping purposes
     */
    struct Transport *(* __stdargs installtransportpl)
	                               (void(* __stdargs routine)(void),char priority);

/* NewScrollingPopUpMenu provides ptr2 and index, which allow the
 * pop up requester to start in the middle instead of always at the
 * top
 */
   struct ListNode *(* __stdargs NewScrollingPopUpMenu)(struct Screen *screen,
                     void *ptr1, void (* __stdargs displayroutine)(void),
	                  short itemheight, short itemwidth, short count,
                     void *ptr2, short index);

/* doubleclick returns a non-zero amount of time if a doubleclick occurs */
   int (* __stdargs doubleclick)(struct IntuiMessage *message);

/* msgbox is a routine to put a requester on the screen with one to
 * three buttons. It returns the number of button from 0 to 2 from the left
 */
    int (* __stdargs msgbox) (char *text,char *b0Text,char *b1Text,
                   char *b2Text,short center,short backcolor,short textcolor,
                   short buttoncolor);
   void (* __stdargs calctrack)(struct Track *track);
    void (* __stdargs removealltool)(struct ToolMaster *toolmaster,char remove);
    void (* __stdargs copyfile)(char *fromname,char *topath);

    long (* __stdargs removetoolfrommacro)(struct MacroTool *macro,
		                          struct ToolMaster *toolmaster,
		                          char remove);
    void (* __stdargs edittool)(struct Tool *tool);
    void (* __stdargs addtoolremover)(long (* __stdargs routine)());
    void (* __stdargs removetoolremover)(long (* __stdargs routine)());

    void (* __stdargs drawslice)(struct RastPort *rp,struct Image *image,
	                   short x,short y,short start,short finish);
    void (* __stdargs drawtool)(struct Tool *tool,struct RastPort *rp,short x,short y,
	                  short h);
    long (* __stdargs checklegaltool)(char *id);   /* for MM tools, check if legal */

    void (* __stdargs settimenow)(long time);	/* Set current time. */
    void (* __stdargs updateobject)(void *object,void *data);
    void (* __stdargs linkobject)(struct Window *window,void *object,void (*routine)());
    void (* __stdargs unlinkobject)(struct Window *window,void *object);
    void (* __stdargs unlinkwindow)(struct Window *window);

    void (* __stdargs installhitlist)(struct Track *track,struct HitName *list);

    void (* __stdargs settempo)(short tempo);
    void (* __stdargs installtempochangelist)(struct TempoChange *);

    struct Transport *(* __stdargs installtransportp)(void (* __stdargs routine)(long,long,long),
                                           char priority);

    long (* __stdargs fastgets)(long,long,long);	/* fgets() */
    long (* __stdargs fastseek)(long,long,long);	/* Fast seek file io. */

    void (* __stdargs adddisplayserver)(void (* __stdargs routine)(void),struct Window *window);
    void (* __stdargs removedisplayserver)(struct Window *window);

    struct Tool *(* __stdargs dragtool)(struct Window *window,
                             struct ToolMaster *toolmaster,
                             struct Tool *copy);
    void (* __stdargs addtoolserver)(struct Tool * (* __stdargs routine)(void),struct Window *window);
    void (* __stdargs removetoolserver)(struct Window *window);

    long (* __stdargs editsysex)(struct Track *track,struct StringEvent *stringevent);
    struct Event *(* __stdargs dupeevent)(struct Event *event); /* Duplicate an event. */

    long (* __stdargs frametotime)(long frame);	/* Convert frame to time. */
    long (* __stdargs timetoframe)(long time);		/* Convert time to frame. */
    long (* __stdargs frametohmsf)(long frame);	/* Convert frame to hmsf. */
    long (* __stdargs hirestotime)(long hires);	/* Hires clock to time. */
    long (* __stdargs timetohires)(long time);		/* Time to hires. */
    long (* __stdargs hirestoframe)(long hires);	/* Hires clock to frame. */
    long (* __stdargs frametohires)(long frame);	/* Frame to hires clock. */	

    void (* __stdargs deletetool)(struct Tool *tool);
    struct Tool *(* __stdargs createtool)(struct ToolMaster *toolmaster,struct Tool *copy);
    long (* __stdargs sizetool)(struct Tool *tool);
    long (* __stdargs savetool)(long file,struct Tool *tool);
    struct Tool *(* __stdargs loadtool)(long file,struct ToolMaster *toolmaster,long size);
    struct ToolMaster *(* __stdargs gettoolmaster)(long id);

/*	Old routines: */
    
    void (* __stdargs processsmpteclock)(struct Event *event);
    void (* __stdargs processmidiclock)(struct Event *event);/* Process a MIDI clock event */
    void (* __stdargs processsysex)();		/* Does nothing. */
    void (* __stdargs processinputevent)(char command);

    long (* __stdargs clearenvironment)(struct Environment *environ,char all);
    long (* __stdargs installenvironment)(struct Environment *environ);
    long (* __stdargs loadsong)(long file,struct Environment *environ);
    long (* __stdargs savesong)(long file,struct Environment *environ);

    struct Transport *(* __stdargs installtransport)(void (* __stdargs routine)(long,long,long));
    void (* __stdargs removetransport)(void (* __stdargs routine)(long,long,long));/* Remove transport handler. */
    void (* __stdargs transportcommand)(unsigned char command,long time);	/* Send command to handlers. */

    void           (* __stdargs qevent)(struct Event *event);	/* Put event in queue. */
    struct Event * (* __stdargs allocevent)(void);			/* Allocate an event. */
    struct Event * (* __stdargs fastallocevent)(void);	/* Allocate an event from interrupt. */
    void           (* __stdargs freeevent)(struct Event *event);/* Free an event. */
    struct Event * (* __stdargs sorteventlist)(struct Event *events);
    void           (* __stdargs freelist)(struct Event *list);	/* Free a list of events. */
    struct Event * (* __stdargs dupelist)(struct Event *list); /* Duplicate a list. */

    struct String *(* __stdargs allocstring)(char *text);
    void           (* __stdargs freestring)(struct String *string);
    void           (* __stdargs replacestring)(struct String **string,char *text);
    void           (* __stdargs dupestring)(struct String **string);
    char *         (* __stdargs stringtext)(struct String *string);

    void           (* __stdargs clearclip)(struct Clip *clip);	/* Clear a clip. */
    void           (* __stdargs dupeclip)(struct Clip *source, struct Clip *destination);
    void           (* __stdargs cutclip)(struct Clip *source, struct Clip *clip,
	                           unsigned long begin, unsigned long end);
    void           (* __stdargs copyclip)(struct Clip *source,struct Clip *clip,
	                            unsigned long begin, unsigned long end);
    void           (* __stdargs pasteclip)(struct Clip *source,struct Clip *clip,
	                             unsigned long begin);
    void           (* __stdargs mixclip)(struct Clip *source,struct Clip *clip,
	                           unsigned long begin,unsigned long end);
    struct Clip *  (* __stdargs loadclip)(long filein);
    long           (* __stdargs saveclip)(long filein,struct Clip *clip);
    long           (* __stdargs clipboard)(long command,struct Clip *clip);

    struct Track * (* __stdargs createtrack)(char new,char in);
    void           (* __stdargs deletetrack)(struct Track *track);	/* Delete a track. */

    char *         (* __stdargs myalloc)(long size,long flags);	/* Internal allocation routine. */
    void           (* __stdargs myfree)(char *item,long size);	/* Internal free routine. */

    long (* __stdargs doscall)(long arg1, ... );			/* Make a DOS command. */
    long (* __stdargs fastopen)(char *name,long mode);	/* Fast file open. */
    long (* __stdargs fastwrite)(long file,char *buffer,long size);
    long (* __stdargs fastread)(long file,char *buffer,long size);
    long (* __stdargs fastclose)(long file);				/* Fast file close. */

    long           (* __stdargs popupkey)(struct Window *window,long was);
    long           (* __stdargs popupnote)(struct Window *window,long was);
    long           (* __stdargs popupoctave)(struct Window *window,long was);
    struct Chord * (* __stdargs popupchord)(struct Chord *old);
    struct Rhythm *(* __stdargs popuprhythm)(struct Rhythm *old);
    struct Chord * (* __stdargs popupscale)(struct Chord *old);

    long (* __stdargs measuretotime)(struct Clip *clip,unsigned short measure);
    long (* __stdargs timetomeasure)(struct Clip *clip,long time);
    long (* __stdargs totalbeatstotime)(struct Clip *clip,unsigned long beats);
    long (* __stdargs timetototalbeats)(struct Clip *clip,unsigned long time);
    void (* __stdargs lengthtostring)(struct Clip *clip,unsigned long time,
                          unsigned long length,char *buffer);
    unsigned long (* __stdargs stringtolength)(struct Clip *clip,unsigned long time,
                                    char *buffer,unsigned long length);
    void (* __stdargs timetostring)(struct Clip *clip,long time,char *buffer);
    unsigned long (* __stdargs stringtotime)(struct Clip *clip,char *buffer,
                                  unsigned long time);
    void          (* __stdargs frametostring)(long frame,char *buffer);
    unsigned long (* __stdargs stringtoframe)(char *buffer,unsigned long frame);
    void          (* __stdargs notetostring)(struct Clip *clip,unsigned short note,
	                               char *string);
    short         (* __stdargs stringtonote)(char *string);

    long               (* __stdargs noteinkey)(struct Clip *clip,struct NoteEvent *note);
    long               (* __stdargs noteinchord)(struct Clip *clip,struct NoteEvent *note);
    long               (* __stdargs noteinrhythm)(struct Clip *clip,struct NoteEvent *note);
    struct KeyEvent *  (* __stdargs timetokey)(struct Clip *clip,unsigned long time);
    struct ChordEvent *(* __stdargs timetochord)(struct Clip *clip,unsigned long time);
    unsigned short     (* __stdargs timetodynamics)(struct Clip *clip,long time);
    struct NoteEvent * (* __stdargs nextrhythmbeat)(struct Clip *clip,long time,
                                         long *eventtime);
    long (* __stdargs scaletotwelve)(struct Clip *clip,unsigned long time,char value);
    long (* __stdargs twelvetoscale)(struct Clip *clip,unsigned long time,char value,
                          char *offset);
    long (* __stdargs random)(long range);		/* Return random number. */

    long (* __stdargs areyousure)(char *text);	/* Put up "are you sure?" requester. */
    void (* __stdargs openwait)(void);			/* Open wait requester. */
    void (* __stdargs closewait)(void);			/* Close wait requester. */

    void (*display)(long refresh);	/* Display main window. */
    struct ListNode *(* __stdargs ScrollingPopUpMenu)(struct Screen *screen,
                                           struct ListNode *list,
                                           void (* __stdargs displayroutine)(void),
                                           short itemheight,short itemwidth,
                                           short count);
    void (* __stdargs DragSlider)(long window,long gadget,long width,long routine,char axis);
    void (* __stdargs DrawSlider)(long window,long gadget,long position,long width,
                       long routine,char axis);
/*	Inovatools 1 routines. */
    long (* __stdargs Itoa)(long integer, char *s,long radix);
    long (* __stdargs Atoi)(char *s,long radix);
    void (* __stdargs RefreshGadget)(struct Window *window,long id);
    struct Gadget *(* __stdargs GetGadget)(struct Window *window,unsigned short id);
/* pass gadgetID for string gadget; returns its StringInfo */
    struct StringInfo *(* __stdargs GetStringInfo)(struct Window *window,unsigned short id);
    void (* __stdargs SetStringInfo)(struct Window *window,unsigned short id,char *string);
    long (* __stdargs GetStringInfoNumber)(struct Window *window,unsigned short id,
                                unsigned short base);
    void (* __stdargs SetStringInfoNumber)(struct Window *window,unsigned short id,
                                long number,long base);
    struct PropInfo *(* __stdargs GetPropInfo)(struct Window *window,unsigned short id);
    void (* __stdargs SetPropInfo)(struct Window *window,unsigned short id,
                        unsigned short hpos,unsigned short vpos);
    struct IntuiMessage *(* __stdargs GetSelectIntuiMessage)(struct Window *window,
                                                  unsigned long type);
    struct IntuiMessage *(* __stdargs GetIntuiMessage)(struct Window *window);
    void (* __stdargs ClearIntuiMessages)(struct Window *window);
    struct Menu *(* __stdargs GetMenu)(struct Menu *menulist,long menunum);
    struct MenuItem *(* __stdargs GetMenuItem)(struct Menu *menu,long menunum,
                                    long itemnum,long subitemnum);
    void (* __stdargs CheckMenuItem)(struct Menu *menu,long menunum,long itemnum,
                          long subitemnum,long on);
    void (* __stdargs EnableMenuItem)(struct Menu *menu,long menunum,long itemnum,
                           long subitemnum,long on);
    void (* __stdargs EnableMenu)(struct Menu *menu,long menunum,long on);
    void (* __stdargs EnableGadget)(struct Window *window,short id,short on);
    void (* __stdargs SelectGadget)(struct Window *window,short id,short on);
    struct NewWindow *(* __stdargs DupeNewWindow)(struct NewWindow *nw);
    void (* __stdargs DeleteNewWindow)(struct NewWindow *nw);
    struct Menu *(* __stdargs DupeMenu)(struct Menu *source);
    void (* __stdargs DeleteMenu)(struct Menu *list);
    void (* __stdargs SendCloseWindow)(struct Window *window);
    long (* __stdargs List_Len)(struct ListItem *list);
    long (* __stdargs List_Position)();
    struct ListItem *(* __stdargs List_Pred)();
    struct ListItem *(* __stdargs List_Index)();
/* these three functions return a pointer to the new list head */
    struct ListItem *(* __stdargs List_Cat)(struct ListItem *l1, struct ListItem *l2);
    struct ListItem *(* __stdargs List_Insert)();
    struct ListItem *(* __stdargs List_Remove)(struct ListItem *head, struct ListItem *bye);

    void (* __stdargs SetScrollBar)(struct ListInfo *info);
    void (* __stdargs DrawList)(struct ListInfo *info);
    void (* __stdargs ScrollList)(struct ListInfo *info);
    struct ListItem *(* __stdargs SizeList)(struct ListInfo *li);
    struct ListItem *(* __stdargs InitListInfo)(struct ListInfo *li);
    void (* __stdargs RemoveListInfo)(struct ListInfo *li);
    struct ListItem *(* __stdargs GetListItem)(struct ListInfo *li);
    void (* __stdargs ClickList)(struct ListInfo *li,short id);
    void (* __stdargs InsertListItem)(struct ListInfo *li,struct ListItem *item);
    struct ListItem *(* __stdargs RemoveListItem)(struct ListInfo *li);
    short (* __stdargs PopUpMenu)(struct Window *window,struct Menu *menu);
    void (* __stdargs DragGadget)(struct Window *window,struct DragInfo *g);
    void (* __stdargs FileName)(char *blankstring,char *prompt,char *ptemplate,
                     struct Screen *screen,short mode,short x,short y);
    struct Window *(* __stdargs FlashyOpenWindow)(struct NewWindow *new);
    void (* __stdargs FlashyCloseWindow)(struct Window *window);
    struct Window *(* __stdargs WhichWindow)(struct Window *window,short x,short y);
    struct ListInfo *(* __stdargs DupeListInfo)(struct ListInfo *listinfo);
    void (* __stdargs DeleteListInfo)(struct ListInfo *listinfo);
    void (* __stdargs RealTimeScroll)(struct ListInfo *li);

/*** embossed library routines ***/

    void (* __stdargs EmbossOn)(struct Window *,short ,short );
    void (* __stdargs EmbossOff)(struct Window *,short );
    void (* __stdargs DrawEmbossed)(struct Window *,short );
    void (* __stdargs EnableEmbossed)(struct Window *,short ,short );
    void (* __stdargs SelectEmbossed)(struct Window *,short ,short );
    void (* __stdargs ModifyEmbossedProp)(struct Window *,short,long,long,long,long,long,long);
    long (* __stdargs DragEmbossedProp)(struct Window *,short );
    long (* __stdargs DrawEmbossedProp)(struct Window *,short );
    long (* __stdargs ShiftEmbossedPropOnce)(struct Window *,short,short,short );
    long (* __stdargs ShiftEmbossedProp)(struct Window *,short,short ,short );
    long (* __stdargs EmbossedPropOn)(struct Window *,short,
	                        long (* __stdargs )(struct Window *,struct Gadget *,long,long),
	                        unsigned short,unsigned short );
    void (* __stdargs FatEmbossedPropOn)(struct Window *,short ,short ,short ,
                              long (* __stdargs )(struct Window *,struct Gadget *,long),
                              unsigned short ,char );
    void (* __stdargs EmbossedPropOff)(struct Window *,short );
    void (* __stdargs FatEmbossedPropOff)(struct Window *,short ,short ,short );
    void (* __stdargs DrawEmbossedRect)(struct RastPort *,short,short,short,short,short);
    void (* __stdargs DrawEmbossedWindow)(struct Window *,short );
    void (* __stdargs EmbossWindowOff)(struct Window *);
    void (* __stdargs EmbossWindowOn)(struct Window *,long ,char *,unsigned short,
	                        unsigned short ,long (* __stdargs )(void),long (*)(void));
    void (* __stdargs EmbossRequestOn)(struct Window *,char *);
    void (* __stdargs RefreshEmbossedWindowFrame)(struct Window *);
    int (* __stdargs SystemGadgets)(struct Window *,long ,struct Gadget *,long);
    void (* __stdargs ModifyWindowProps)(struct Window *,unsigned long ,unsigned long ,
	             unsigned long ,unsigned long ,unsigned long ,unsigned long );
    void (* __stdargs EmbossedWindowTitle)(struct Window *,char *);
	/* writes length chars into window's title bar using RastPort's font */
    void (* __stdargs EmbossedWindowText)(struct Window *window,char *text,short length);
	/* sets text for gadget in window to name using B&P's 5-pixel font */
    void (* __stdargs setsmalltext)(struct Window *window,short gadid,char *name);
    short (* __stdargs fitsmalltext)(char *text,short length);
    void (* __stdargs usetopazfont)(struct RastPort *rp);
	/* sets RastPort font to B&P's 5-pixel font */
    void (* __stdargs usesmallfont)(struct RastPort *rp);
};

/* bits for "globes" assigned in main.c */
#define GLOBES_USEASLREQUEST 1	/* use 2.0 asl requester */
#define GLOBES_INTUITION37	2		/* 2.0 intuition */

/* macros for saving directory locations */
#define FILENAME_TOOL	0
#define FILENAME_ACCESS	1
#define FILENAME_SONG	2
#define FILENAME_TRACK	3
#define FILENAME_GROUP	4
#define FILENAME_TEMPO	5
#define FILENAME_READ	6
#define FILENAME_WRITE	7
#undef FILENAME_MAX
#define FILENAME_MAX	8
