/**
* barsMacros.h -- handy Bars&Pipes macros
*  1991 Another Option   All rights reserved.
**/

/*==========================================================================
|    DATE    | WHAT
============================================================================*/
/* 19 Aug 91 | created
*/

#ifndef BARS_MACROS_H
#define BARS_MACROS_H

#ifndef BARS_H
 #include "bars.h"
#endif

#ifndef EXEC_TYPES_H
 #include <exec/types.h>
#endif

/* make a B&P tool id without digging up an ASCII/hex chart...
   To use this, #define a four-character THIS_TOOL_NAME string in your
   source.  When you need to reference your tool id (e.g., when initializing
   a ToolMaster), just refer to THIS_TOOL_ID.
*/
#define MakeToolID(s) ( (*(UBYTE *)(s)<<24) + (*(UBYTE *)(s+1)<<16) + \
                        (*(UBYTE *)(s+2)<<8) + *(UBYTE *)(s+3) )

#define THIS_TOOL_ID MakeToolID(THIS_TOOL_NAME)


/**** some file-writing macros ****/
/* write a byte */
#define writebyte(f,c) (functions->fastwrite(f,(char *)(&(c)),1) == -1)

/* read & verify that it happened */
#define readv(f,d,l) ( functions->fastread((f),(char *)(&(d)),(l)) != -1 )

/* write & verify that it happened */
#define writev(f,d,len) ( functions->fastwrite((f),(char *)(&(d)),(len)) != -1 )


/*
* some stuff left out of bars.h
*/
/* bits for b&p file operations */
#define FILES_DELETE  1
#define FILES_OPEN    2
#define FILES_SAVE    4
#define FILES_TEST    8
#define FILES_TYPE   16
#define FILES_PATH   32
/* b&p clipboard ops */
#ifndef CLIP_GET
 #define CLIP_GET 1
 #define CLIP_PUT 2
#endif

/* embossed window gadget IDs */
#ifndef EMBOSSED_CLOSE
 #define EMBOSSED_CLOSE 0xFFF
 #define EMBOSSED_DRAG  0xFFE
// #define EMBOSSED_ZOOM  0xFFD /* unverified, but logical */
 #define EMBOSSED_BACK  0xFFC
 #define EMBOSSED_FRONT 0xFFB
#endif

/* default pen colors */
#define BP_BLACK  0
#define BP_WHITE  1
#define BP_LGREY  2
#define BP_DGREY  3
#define BP_BLUE   4
#define BP_PURPLE 5
#define BP_RED    6
#define BP_YELLOW 7

/* offsets for pot clicker gadget IDs */
#ifndef GDOWN
 #define GDOWN 100
#endif
#ifndef GUP
 #define GUP 200
#endif

/* ExecMessage.mn_Length set to this means sysex was read; edittool() should 
   refresh gadgets.
*/
#ifndef SECRET_MESSAGE
 #define SECRET_MESSAGE 9999
#endif

/*** CLANDESTINE use of a tool.touched spare bit ***/
/* set this if it is ok to redraw a window (i.e., gadgetry exists) */
#define TOUCH_REFRESHOK 16
/* macros to manage the bit -- use pointer to Tool */
/* invoke this in edittool() just before going into the event loop */
#define SetRefreshOK(t) ((t)->touched |= TOUCH_REFRESHOK)
/* invoke this on a close window message */
#define ResetRefreshOK(t) ((t)->touched &= ~TOUCH_REFRESHOK)
/* and to test it -- invoke before redrawing window outside of edittool() */
#define RefreshOK(t) ((t)->touched & TOUCH_REFRESHOK)


/*
   B&P MIDI event types -- use pointer to Event
*/
/* no channel numbers in B&P status bytes so masking is unnecessary */
//#define stbyte(e)       ( (e)->status & 0xF0 )
#define stbyte(e)       ( (e)->status )

/* checking byte2 isn't necessary for B&P
#define IsNoteOn(e)     ( (stbyte(e) == MIDI_NOTEON) && (e)->byte2 )
#define IsNoteOff(e)    ( (stbyte(e) == MIDI_NOTEOFF) ||\
                         ((stbyte(e) == MIDI_NOTEON) && !(e)->byte2) )
*/
#define IsNoteOn(e)     ( e->status == MIDI_NOTEON )
#define IsNoteOff(e)    ( e->status == MIDI_NOTEOFF )
#define IsPolyTouch(e)  ( stbyte(e) == MIDI_PTOUCH )
// do I really care if it's really a valid cc#? donthinso
//#define IsCtrlChg(e)    ( (stbyte(e) == MIDI_CCHANGE) && ((e)->byte1 < 121) )
#define IsCtrlChg(e)    ( stbyte(e) == MIDI_CCHANGE )
#define IsPgmChg(e)     ( stbyte(e) == MIDI_PCHANGE )
#define IsMonoTouch(e)  ( stbyte(e) == MIDI_MTOUCH )
#define IsPitchBend(e)  ( stbyte(e) == MIDI_PBEND )
#define IsNote(e)       ( IsNoteOn(e) || IsNoteOff(e) )
#define IsSwitch(e)     ( IsCtrlChg(e) && ((e)->byte1 >= 64) )
#define IsContinuous(e) ( IsCtrlChg(e) && ((e)->byte1 < 64) )
#define IsSysEx(e)      ( stbyte(e) == MIDI_SYSX )

/* test a byte to see if a switch is on or off */
#define SwitchOn(b)  ( (b) > 63 )
#define SwitchOff(b) ( (b) < 64 )

/* message types */
#define IsChannelMsg(e)  ((stbyte(e) >= MIDI_NOTEOFF) && (stbyte(e) <= MIDI_PBEND))
#define IsControlMsg(e)  ((stbyte(e) >= MIDI_PTOUCH) && (stbyte(e) <= MIDI_PBEND))
#define IsModeMsg(e)     ((stbyte(e) == 0xB0))
#define IsCommonMsg(e)   (stbyte(e) == 0xF0)
#define IsRealTimeMsg(e) ((e)->status >= 0xF8)

/* specific channel mode messages */
#define IsAllOff(e)      (IsModeMsg(e) && ((e)->byte1==0x7B))
#define IsAllCtrlOff(e)  (IsModeMsg(e) && ((e)->byte1==0x79))
#define IsSustainOff(e)  (IsModeMsg(e) && ((e)->byte1==0x40))

/*
   Event types -- use pointer to Event or NoteEvent
*/
#define EventType(e)     ((e)->type & 0x0F)
#define IsPadEdit(e)     ((e)->type & EVENT_PADEDIT)
#define IsRealTime(e)    (!IsPadEdit(e))
#define IsBranching(e)   ((e)->type & EVENT_BRANCH)
#define IsPlayOnly(e)    ((e)->type & EVENT_PLAYONLY)
#define IsVoice(e)       (EventType(e) == EVENT_VOICE)
#define IsSysx(e)        (EventType(e) == EVENT_SYSX)
#define IsLyric(e)       (EventType(e) == EVENT_LYRIC)
#define IsTimeSig(e)     (EventType(e) == EVENT_TIMESIG)
#define IsKey(e)         (EventType(e) == EVENT_KEY)
#define IsChord(e)       (EventType(e) == EVENT_CHORD)
#define IsRhythm(e)      (EventType(e) == EVENT_RHYTHM)
#define IsDynamics(e)    (EventType(e) == EVENT_DYNAMICS)
#define IsSeed(e)        (EventType(e) == EVENT_SEED)

/*
	sundries
*/
#define FreeStringEvent(e) { functions->freestring((e)->string);\
                             functions->freeevent((struct Event *)(e)); }


/*** more MIDI ***/

/* standard controllers */
/* Continuous 14 bit - MSB: 0-1F, LSB: 20-3F */
#define MIDI_ModWheel     0x01
#define MIDI_Breath       0x02
#define MIDI_Foot         0x04
#define MIDI_PortaTime    0x05
#define MIDI_DataEntry    0x06
#define MIDI_Volume       0x07
#define MIDI_Balance      0x08
#define MIDI_Pan          0x0A
#define MIDI_Expression   0x0B
#define MIDI_General1     0x10
#define MIDI_General2     0x11
#define MIDI_General3     0x12
#define MIDI_General4     0x13

/* Continuous 7 bit (switches: 0-3F=off, 40-7F=on) */
#define MIDI_Sustain      0x40
#define MIDI_Porta        0x41
#define MIDI_Sustenuto    0x42
#define MIDI_SoftPedal    0x43
#define MIDI_Hold2        0x45
#define MIDI_General5     0x50
#define MIDI_General6     0x51
#define MIDI_General7     0x52
#define MIDI_General8     0x53
#define MIDI_ExtDepth     0x5B
#define MIDI_TremoloDepth 0x5C
#define MIDI_ChorusDepth  0x5D
#define MIDI_CelesteDepth 0x5E
#define MIDI_PhaserDepth  0x5F

/* Parameters */
#define MIDI_DataIncr     0x60
#define MIDI_DataDecr     0x61

/* misc */
#define MIDI_BendCntr 0x2000 /* Pitch Bend center position as a 14 bit word */
#define MIDI_CtrlCntr 64     /* Center value for controllers like Pan */

/*** macros ***/
/* pack high/low bytes of a word into MIDI format (7/14 bit math) */
#define MIDIHiByte(word) ( (word) >> 7 & 0x7f )
#define MIDILoByte(word) ( (word) & 0x7f )

/* unpack 2 MIDI bytes into a word (7/14 bit math) */
#define MIDIWord(hi,lo) ( ((hi) & 0x7f) << 7 | ((lo) & 0x7f) )

/*** primitive debugging poop ***/
#define dumpfev(s,e) dprintf("%s @ 0x%8lX next=0x%8lX\ntime=%10ld type=%3ld tool=0x%8lX branch=0x%8lX",\
                      s,(e),(e)->next,(e)->time,(e)->type,(e)->tool,(e)->tool->branch)


#define dumpev(s,e) dprintf("%s @ 0x%8X next=0x%8X\ntime=%8ld type=%3ld tool=%8X",\
                      s,(e),(e)->next,(e)->time,(e)->type,(e)->tool)

#define dumpevent(s,e) dumpev(s,e);dprintf("\n")

#define dumpmidiev(s,e) dumpev(s,e);dprintf("\nmidi=0x%2X 0x%2X 0x%2X\n",\
                                           (e)->status,(e)->value,(e)->velocity)

#define dumpmidi(s,e) dprintf("%s @0x%8X 0x%2X 0x%2X 0x%2X\n",\
                               s,(e)->time,(e)->status,(e)->byte1,(e)->byte2)

#define dumpnote(s,e) dumpmidiev(s,((struct NoteEvent *)e));\
                      dprintf("duration=%8ld\n",((struct NoteEvent *)(e))->duration)

#define dumpdynamics(s,e) dprintf("%s %10ld 0x%2X\n",s,(e)->time,(e)->value)


#endif /*!BARS_MACROS_H*/
